#include <jacket.h>

static __global__ void kernel(int nx, int ny, float *d_out, float *d_in)
{
    int x = blockIdx.x * blockDim.x + threadIdx.x;
    int y = blockIdx.y * blockDim.y + threadIdx.y;
    d_out[y*nx + x] = 0;
}


err_t jktFunction(int nlhs, mxArray *plhs[], int nrhs, mxArray *prhs[])
{
    if (nrhs != 1)
        return err("expects one input");
    if (nlhs > 1)
        return err("expects one output");
    mxArray *m_in = prhs[0];
    if (!jkt_garray(m_in))
        return err("expects gsinle input");
    if (jkt_gfor(m_in))
        return err("gfor not supported");
    if (jkt_class(m_in) != mxSINGLE_CLASS)
        return err("expects single-precision input");
    if (jkt_complex(m_in))
        return err("expects real-valued input");
    const mwSize *dims;
    int ndims = jkt_dims(m_in, &dims);
    if (ndims != 2 || dims[0] <= 1 || dims[1] <= 1)
        return err("expects 2D matrix");
    if (dims[0] % 16 != 0 || dims[1] % 16 != 0)
        return err("expects each dimension is multiple of four");

    mxArray *m_out = plhs[0] = jkt_new_array(ndims, dims, mxSINGLE_CLASS, false);

    // get pointers to input and output
    float *d_in, *d_out;
    TRY(jkt_mem((void **)&d_in, m_in));
    TRY(jkt_mem((void **)&d_out, m_out));

    // launch CUDA kernel
    dim3 blocks(dims[0]/4, dims[1]/4);
    dim3 threads(4,4);
    kernel<<<blocks,threads>>>(dims[0], dims[1], d_out, d_in);

    CUDA(cudaThreadSynchronize());
    cudaError_t e = cudaGetLastError();
    if (e != cudaSuccess)
        return err(cudaGetErrorString(e));

    return errNone;
}
