#include <jacket.h>

static __device__ void kernel(int by, int nx, int ny, float *d_out, float *d_in)
{
    int x = blockIdx.x * blockDim.x + threadIdx.x;
    int y =    by      * blockDim.y + threadIdx.y;

    int i = 0;
    float v[9] = {0,0,0,0,0,0,0,0,0}; // zero padding
    for (int xx = x - 1; xx <= x + 1; xx++) {
        for (int yy = y - 1; yy <= y + 1; yy++) {
            if (0 <= xx && xx < nx && 0 <= yy && yy < ny) // boundaries
                v[i++] = d_in[yy*nx + xx];
        }
    }

    // bubble-sort
    for (int i = 0; i < 9; i++) {
        for (int j = i + 1; j < 9; j++) {
            if (v[i] > v[j]) { // swap?
                float tmp = v[i];
                v[i] = v[j];
                v[j] = tmp;
            }
        }
    }

    // pick the middle one
    d_out[y*nx + x] = v[4];
}
static __global__ void gfor(int ngfor, int nx, int ny, float *d_out, float *d_in)
{
    // determine block and tile indices
    int by = blockIdx.y / ngfor; // adjusted blockIdx.y
    int id = blockIdx.y % ngfor;

    // move to correct tiles
    d_in += id * nx*ny; d_out += id * nx*ny;
    kernel(by, nx, ny, d_out, d_in);
}
static __global__ void normal(int nx, int ny, float *d_out, float *d_in)
{
    kernel(blockIdx.y, nx, ny, d_out, d_in);
}


err_t jktFunction(int nlhs, mxArray *plhs[], int nrhs, mxArray *prhs[])
{
    mxArray *m_in = prhs[0];
    const mwSize *dims;
    int ndims = jkt_dims(m_in, &dims);

    // determine gfor tiles
    int ngfor = jkt_gfor(m_in);
    mxArray *m_out = plhs[0] = jkt_new_array_N(ndims, dims, mxSINGLE_CLASS, false, ngfor);

    // get pointers to input and output
    float *d_in, *d_out;
    TRY(jkt_mem((void **)&d_in, m_in));
    TRY(jkt_mem((void **)&d_out, m_out));

    // launch CUDA kernel
    dim3 blocks(dims[0]/16, dims[1]/16);
    dim3 threads(16,16);
    if (ngfor) {
        blocks.y *= ngfor; // tile out in last dimension
        gfor<<<blocks,threads>>>(ngfor, dims[0], dims[1], d_out, d_in);
    } else {
        normal<<<blocks,threads>>>(dims[0], dims[1], d_out, d_in);
    }

    return errNone;
}
