#include <jacket.h>

__global__ static void kernel(int nx, int ny, float *d_out, float *d_in)
{
    int x = blockIdx.x * blockDim.x + threadIdx.x;
    int y = blockIdx.y * blockDim.y + threadIdx.y;

    int i = 0;
    float v[9] = {0,0,0,0,0,0,0,0,0}; // zero padding
    for (int xx = x - 1; xx <= x + 1; xx++) {
        for (int yy = y - 1; yy <= y + 1; yy++) {
            if (0 <= xx && xx < nx && 0 <= yy && yy < ny) // boundaries
                v[i++] = d_in[yy*nx + xx];
        }
    }

    // bubble-sort
    for (int i = 0; i < 9; i++) {
        for (int j = i + 1; j < 9; j++) {
            if (v[i] > v[j]) { /* swap? */
                float tmp = v[i];
                v[i] = v[j];
                v[j] = tmp;
            }
        }
    }

    // pick the middle one
    d_out[y*nx + x] = v[4];
}


err_t jktFunction(int nlhs, mxArray *plhs[], int nrhs, mxArray *prhs[])
{
    mxArray *m_in = prhs[0];
    const mwSize *dims, ndims = jkt_dims(m_in, &dims);

    mxArray *m_out = plhs[0] = jkt_new_array(ndims, dims, mxSINGLE_CLASS, false);

    int block_size = (nrhs == 2) ? mxGetScalar(prhs[1]) : 16;

    // get pointers to input and output
    float *d_in, *d_out;
    TRY(jkt_mem((void **)&d_in,  m_in));
    TRY(jkt_mem((void **)&d_out, m_out));

    // launch CUDA kernel
    dim3 blocks(dims[0]/block_size, dims[1]/block_size);
    dim3 threads(block_size,block_size);
    kernel<<<blocks,threads>>>(dims[0], dims[1], d_out, d_in);

    return errNone;
}
