#include <jacket.h>

// Exchange trick: Morgan McGuire, ShaderX 2008
#define s2(a,b)            { float tmp = a; a = min(a,b); b = max(tmp,b); }
#define mn3(a,b,c)         s2(a,b); s2(a,c);
#define mx3(a,b,c)         s2(b,c); s2(a,c);

#define mnmx3(a,b,c)       mx3(a,b,c); s2(a,b);                               // 3 exchanges
#define mnmx4(a,b,c,d)     s2(a,b); s2(c,d); s2(a,c); s2(b,d);                // 4 exchanges
#define mnmx5(a,b,c,d,e)   s2(a,b); s2(c,d); mn3(a,c,e); mx3(b,d,e);          // 6 exchanges
#define mnmx6(a,b,c,d,e,f) s2(a,d); s2(b,e); s2(c,f); mn3(a,b,c); mx3(d,e,f); // 7 exchanges

#define IN(X,Y)  ((0 <= (X) && (X) < nx && 0 <= (Y) && (Y) < ny) ? d_in[(Y)*nx+(X)] : 0)

__global__ static void kernel(int nx, int ny, float *d_out, float *d_in)
{
    int x = blockIdx.x * blockDim.x + threadIdx.x;
    int y = blockIdx.y * blockDim.y + threadIdx.y;

    // pull top six from image
    float v[6] = { IN(x-1, y-1), IN(x  , y-1), IN(x+1, y-1),
                   IN(x-1, y  ), IN(x  , y  ), IN(x+1, y  ) };

    // with each pass, remove min and max values and add new value
    mnmx6(v[0], v[1], v[2], v[3], v[4], v[5]);
    v[5] = IN(x-1, y+1); // add new contestant
    mnmx5(v[1], v[2], v[3], v[4], v[5]);
    v[5] = IN(x  , y+1); // add new contestant
    mnmx4(v[2], v[3], v[4], v[5]);
    v[5] = IN(x+1, y+1); // add last contenstant
    mnmx3(v[3], v[4], v[5]);

    // pick the middle one
    d_out[y*nx + x] = v[4];
}


err_t jktFunction(int nlhs, mxArray *plhs[], int nrhs, mxArray *prhs[])
{
    mxArray *m_in = prhs[0];
    const mwSize *dims, ndims = jkt_dims(m_in, &dims);

    mxArray *m_out = plhs[0] = jkt_new_array(ndims, dims, mxSINGLE_CLASS, false);

    int block_size = (nrhs == 2) ? mxGetScalar(prhs[1]) : 16;

    // get pointers to input and output
    float *d_in, *d_out;
    TRY(jkt_mem((void **)&d_in,  m_in));
    TRY(jkt_mem((void **)&d_out, m_out));

    // launch CUDA kernel
    dim3 blocks(dims[0]/block_size, dims[1]/block_size);
    dim3 threads(block_size,block_size);
    kernel<<<blocks,threads>>>(dims[0], dims[1], d_out, d_in);

    return errNone;
}
