#include <jacket.h>

texture<float> tex_in;
#define IN(x,y)    tex1Dfetch(tex_in, (y)*nx + (x) + off)

static __global__ void kernel(int nx, int ny, float *d_out, int off)
{
    int x = blockIdx.x * blockDim.x + threadIdx.x;
    int y = blockIdx.y * blockDim.y + threadIdx.y;

    int i = 0;
    float v[9] = {0,0,0,0,0,0,0,0,0}; // zero padding
    for (int xx = x - 1; xx <= x + 1; xx++) {
        for (int yy = y - 1; yy <= y + 1; yy++) {
            if (0 <= xx && xx < nx && 0 <= yy && yy < ny) // boundaries
                v[i++] = IN(xx,yy);
        }
    }

    // bubble-sort
    for (int i = 0; i < 9; i++) {
        for (int j = i + 1; j < 9; j++) {
            if (v[i] > v[j]) { // swap?
                float tmp = v[i];
                v[i] = v[j];
                v[j] = tmp;
            }
        }
    }

    // pick the middle one
    d_out[y*nx + x] = v[4];
}


err_t jktFunction(int nlhs, mxArray *plhs[], int nrhs, mxArray *prhs[])
{
    mxArray *m_in = prhs[0];
    const mwSize *dims;
    int ndims = jkt_dims(m_in, &dims);

    mxArray *m_out = plhs[0] = jkt_new_array(ndims, dims, mxSINGLE_CLASS, false);

    int block_size = nrhs == 2 ? mxGetScalar(prhs[1]) : 16;

    // get pointers to input and output
    float *d_in, *d_out;
    TRY(jkt_mem((void **)&d_in, m_in));
    TRY(jkt_mem((void **)&d_out, m_out));

    // bind texture
    size_t off;
    CUDA(cudaBindTexture(&off, tex_in, d_in, dims[0]*dims[1]*sizeof(float)));

    // launch CUDA kernel
    dim3 blocks(dims[0]/block_size, dims[1]/block_size);
    dim3 threads(block_size,block_size);
    kernel<<<blocks,threads>>>(dims[0], dims[1], d_out, off);

    CUDA(cudaUnbindTexture(tex_in));

    return errNone;
}
