% NOTE: this uses the new gsync/geval
% if you are running with v1.2.2, then uncomment the next two lines to run
% gsync = @gforce;
% geval = @gforce;

clear

% data dimensions
n = 512;
% timing repetitions
reps = 1000;

A = gones(n);

% warm up and ensure functionally correct
a = medfilt2_cpu(A);
a_ = medfilt2_cpu_exch(A);   assert(logical(all(a(:) == a_(:))))
a_ = medfilt2_gmem(A);       assert(logical(all(a(:) == a_(:))))
a_ = medfilt2_gmem_exch(A);  assert(logical(all(a(:) == a_(:))))
a_ = medfilt2_gmem_r9(A);    assert(logical(all(a(:) == a_(:))))
a_ = medfilt2_exch(A);       assert(logical(all(a(:) == a_(:))))

gsync % block

tic
  for r = 1:reps
    a = medfilt2_cpu(A);
  end
cpu = toc / reps

tic
  for r = 1:reps
    a = medfilt2_cpu_exch(A);
  end
cpu_exch = toc / reps

gsync % block until GPU ready
tic
  for r = 1:reps
    a = medfilt2_gmem(A);
    geval(a) % ensure executed
  end
gsync % wait until GPU done
gmem = toc / reps

gsync % block until GPU ready
tic
  for r = 1:reps
    a = medfilt2_gmem_exch(A);
    geval(a) % ensure executed
  end
gsync % wait until GPU done
gmem_exch = toc / reps

gsync % block until GPU ready
tic
  for r = 1:reps
    a = medfilt2_gmem_r9(A);
    geval(a) % ensure executed
  end
gsync % wait until GPU done
gmem_r9 = toc / reps

gsync % block until GPU ready
tic
  for r = 1:reps
    a = medfilt2_exch(A);
    geval(a) % ensure executed
  end
gsync % wait until GPU done
exch = toc / reps
